'use strict';
/* globals dS, Foundation */

(function ($) {

    // shortcuts
    var i18n;
    var ajaxRequest;
    var callout;

    // var filterGtin = [ '7640156792539' ];
    var filterVdcHardwareName = 'Homematic Controller';
    var wallButtonName = 'Wall Button';

    var vdcPresentTimeout = 5000;
    var checkInterval = 3000;
    var checkLimit = 5;
    var checkCount;
    var retryLimit = 3;
    var retryDelay = 1000;
    var reregisterDelay = 7500;

    var vdcToken = null;
    // var deviceStateDescriptions = {};
    var dsmuids = [];

    var cloudAvailable = null;

    var $deviceTable;
    var $deviceTableWorkingCopy;
    var $deviceTableHeader;
    var $rediscoverButton;
    var $searchingForDevicesPopup;
    var $remoteConnectivityPopup;
    var $editIpDialog;
    var $editWallButtonTypeDialog;
    var $noDevicesFoundPopup;
    var $noControllersPopup;

    /**
     * SET UP THE APP
     */

    function buildEnvironment () {
        i18n = dS.i18n;
        ajaxRequest = dS.utils.ajaxRequest;
        callout = dS.utils.showStatusCallout;

        $deviceTable = $('#Devices');
        $deviceTableWorkingCopy = $deviceTable.clone();
        $deviceTableHeader = $deviceTable.find('thead .sub-header');
        $rediscoverButton = $('#button_rediscover_devices');
        $searchingForDevicesPopup = $('#SearchingForDevices');
        $remoteConnectivityPopup = $('#RemoteConnectivity');
        $editIpDialog = $('#EditIp');
        $editWallButtonTypeDialog = $('#EditWallButtonType');
        $noDevicesFoundPopup = $('#NoDevicesFound');
        $noControllersPopup = $('#NoControllersFound');

        Foundation.Abide.defaults.validators.ipPart = function ($element, required) {
            var value = $element.val();
            if (!required && value === '') {
                return true;
            }
            value = parseInt(value, 10);
            return (!isNaN(value) && value >= 0 && value <= 255);
        };

        dS.utils.detectFeed();
    }

    function translate () {
        i18n.buildUpLang([ 'outdated_browser_fragment', 'content_fragment', 'missing_remote_connectivity', 'bottom_fragment' ]);
        $deviceTableHeader.find('[id]').removeAttr('id');
        $('body').addClass('loaded');
    }

    /**
     * DEVICES LISTING
     */

    function tableBuildingCompleted (replaceWithWorkingCopy) {
        if (replaceWithWorkingCopy) {
            $deviceTable.html($deviceTableWorkingCopy.html());
        }
        $rediscoverButton.setState('default');
        $searchingForDevicesPopup.close();
    }

    function queryForIp (dsmuid, retrying) {
        var $ipCell = $deviceTable.find('[data-dsmuid="' + dsmuid + '"] .ip .rel');

        ajaxRequest('/json/property/vdcquery', {
            query: 'getProperty(deviceId=' + dsmuid + ')/homematicHostIp(*)'
        }, function (data) {
            if (data.ok && data.result && data.result.elements && Object.prototype.toString.call(data.result.elements) === '[object Array]') {
                if (typeof data.result.elements[0] !== 'undefined') {
                    var ip = data.result.elements[0].value;
                    if (ip) {
                        ip = ip.match(/([0-9]{1,3}\.){3}[0-9]{1,3}(\:[0-9]+)?$/g);
                        $ipCell.html(ip && typeof ip[0] !== 'undefined' ? 'IP ' + ip[0] : dS.utils.faError + ' <span>' + i18n._('IP malformed') + '</span>');
                    } else {
                        $ipCell.html(i18n._('IP not assigned'));
                    }
                } else {
                    $ipCell.html(i18n._('IP not assigned'));
                }
            } else {
                $ipCell.html(dS.utils.faError + ' <span>' + i18n._('IP could not be read') + '</span>');
            }
            $ipCell.setState('halted');
        }, function () {
            if (retrying >= retryLimit) {
                $ipCell
                    .setState('halted')
                    .html(dS.utils.faError + ' <span>' + i18n._('IP could not be read') + '</span>');
            } else {
                retrying = (retrying ? retrying + 1 : 1);
                setTimeout(function () {
                    queryForIp(dsmuid, retrying);
                }, retryDelay);
            }
        });
    }

    function insertSubheader ($dsmTr) {
        var $subheader = $deviceTableHeader.clone();
        $subheader.removeClass('hide');
        $dsmTr.after($subheader);
    }

    function insertDevice (device) {
        var $tr = $('<tr' + (device.present ? '' : ' class="absent"') + '>');
        $tr
            .attr('data-dsuid', device.dSUID)
            .attr('data-display-id', device.DisplayID)
            .attr('data-parent-dsmuid', device.DSMeterDSUID)
            .html(' <td data-before="' + i18n._('Name') + '" class="name">' + (device.name ? device.name : '-') + '</td>' +
                '   <td data-before="' + i18n._('Model') + '">' + device.HWInfo + '</td>' +
                '   <td data-before="' + i18n._('Status') + '" class="status state_' + device.dSUID + ' flex" colspan="2">' +
                '       <em>' + i18n._(device.present ? 'connected' : 'disconnected') + '</em>' +
                '</td>');   // don't put whitespace here before </td> (it'll mess up :empty styling)

        var $dsmTr = $deviceTableWorkingCopy.find('[data-dsmuid="' + device.DSMeterDSUID + '"]');
        var $deviceTrs = $dsmTr.nextAll('[data-parent-dsmuid="' + device.DSMeterDSUID + '"]');
        if ($deviceTrs.length) {
            $deviceTrs.last().after($tr);
        } else {
            $dsmTr.after($tr);
            insertSubheader($dsmTr);
            $dsmTr.removeClass('no-devices');
        }
    }

    function insertDevices (devices) {
        for (var dsuid in devices) {
            if (devices.hasOwnProperty(dsuid) && devices[dsuid] && devices[dsuid].DSMeterDSUID && dsmuids.indexOf(devices[dsuid].DSMeterDSUID) !== -1) {
                insertDevice(devices[dsuid]);
            }
        }
        tableBuildingCompleted(true);

        if (!Object.keys(devices).length) {
            $noDevicesFoundPopup.open();
        }

        for (var i = 0, len = dsmuids.length; i < len; i++) {
            queryForIp(dsmuids[i]);
        }
    }

    function queryForDevices () {

        function errorCallback () {
            $deviceTable.setTableState(false);
            tableBuildingCompleted(false);
            callout('error', i18n._('Checking for devices has failed. Please refresh the page in a few minutes to try again.'));
        }

        ajaxRequest('/json/property/query2', {
            query: '/apartment/zones/zone0/devices/*(dSUID,name,HWInfo,DSMeterDSUID,DisplayID,present)/productInfo(EAN,ConfigLink)'
        }, function (data) {
            if (data.ok && data.result) {
                var devices = data.result;
                // skip non-Homematic devices
                for (var dsuid in devices) {
                    if (devices.hasOwnProperty(dsuid)) {
                        // if (!devices[dsuid].productInfo ||
                        //     !devices[dsuid].productInfo.EAN ||
                        //     filterGtin.indexOf(devices[dsuid].productInfo.EAN) === -1) {
                        if (!devices[dsuid].DSMeterDSUID ||
                            dsmuids.indexOf(devices[dsuid].DSMeterDSUID) === -1) {
                            delete devices[dsuid];
                        }
                    }
                }
                $deviceTable.setTableState('success');
                insertDevices(devices);
            } else {
                errorCallback();
            }
        }, function () {
            errorCallback();
        });
    }

    function insertDsm (dsm) {
        var $tr = $('<tr class="dark no-devices">');
        $tr
            .attr('data-dsmuid', dsm.dSUID)
            .html(' <td data-before="' + i18n._('Name') + '" class="name"><span class="rel">' + dsm.name + '</span></td>' +
                '   <td data-before="' + i18n._('IP') + '" class="ip"><span class="rel in-progress">&nbsp;</span></td>' +
                '   <td class="flex"></td>' +
                '   <td class="actions text-center large-text-right">' +
                '       <button class="vdsm-edit button secondary" data-open="EditIp">' + i18n._('Set IP') + '</button>' +
                '   </td>');

        var $currentIp = $deviceTable.find('[data-dsmuid="' + dsm.dSUID + '"] .ip');
        if ($currentIp.length) {
            $tr.find('.ip')
                .html($currentIp.html())
            .find('.rel')
                .addClass('in-progress');
        }

        $deviceTableWorkingCopy.append($tr);
    }

    function insertDsms (dsms) {
        for (var dsmuid in dsms) {
            if (dsms.hasOwnProperty(dsmuid)) {
                insertDsm(dsms[dsmuid]);
            }
        }
    }

    function queryForDsms () {

        function errorCallback () {
            $noControllersPopup.close();
            tableBuildingCompleted(false);
            $deviceTable.setTableState('error');
            callout('error', i18n._('Checking for controllers has failed. Please refresh the page in a few minutes to try again.'));
        }

        ajaxRequest('/json/property/query2', {
            query: '/apartment/dSMeters/*(dSUID,name,hardwareName)'
        }, function (data) {
            dsmuids = [];
            if (data && data.result && typeof data.result === 'object') {
                var dsms = data.result;
                for (var dsmuid in dsms) {
                    if (dsms[dsmuid].hardwareName.indexOf(filterVdcHardwareName) !== -1) {
                        dsmuids.push(dsmuid);
                    } else {
                        delete dsms[dsmuid];
                    }
                }
                if (Object.keys(dsms).length) {
                    $noControllersPopup.close();
                    insertDsms(dsms);
                    queryForDevices();
                } else {
                    $rediscoverButton.setState('halted');
                    $noControllersPopup.open();

                    setTimeout(queryForDsms, vdcPresentTimeout);
                }
            } else {
                errorCallback();
            }
        }, function () {
            errorCallback();
        }, function () {
            $rediscoverButton.setState('in progress');
        });
    }

    function buildTable (initial) {
        $deviceTableWorkingCopy.html($deviceTable.html()).find('[data-dsmuid], [data-dsuid], tbody .sub-header').remove();
        $deviceTable.setTableState('in progress');
        if (initial !== true) {
            $rediscoverButton.setState('in progress');    // different behaviour between first page load and refresh button click
        }

        setTimeout(function () {
            // dS.utils.abortRequests();    // @TODO - make requests cancellable by criteria
            queryForDsms();
        }, 0);
    }

    /**
     * VDC SET IP
     */

    function formModified ($form) {
        var $inputs = $form.find('input, select');
        var $input;
        var inputType;
        for (var i = 0, len = $inputs.length; i < len; i++) {
            $input = $inputs.eq(i);
            inputType = $input.attr('type');
            if (inputType === 'checkbox' || inputType === 'radio') {
                if ($input.is(':checked') && $input.attr('data-initial-val') === 'false' ||
                    !$input.is(':checked') && $input.attr('data-initial-val') === 'true') {
                    return true;
                }
            } else {
                if ($input.val() !== $input.attr('data-initial-val')) {
                    return true;
                }
            }
        }
        return false;
    }

    function openIpDialog () {
        // when, opened, clean up the dialog back to its "initial' state"
        var $tr = $(this).closest('[data-dsmuid]');
        var dsmuid = $tr.attr('data-dsmuid');
        var $form = $editIpDialog.find('form');
        $editIpDialog
            .attr('data-dsmuid', dsmuid)
            .setDialogState('default');
        $form
            .setFormState('default')
            .foundation('resetForm');   // clear form and validation
        // fill the form in a modal with to-be-edited dsm data
        var ip = $tr.find('.ip .rel').text().slice(3);
        if (ip.match(/([0-9]{1,3}\.){3}[0-9]{1,3}$/g)) {
            var ipParts = ip.split('.');
            for (var i = 1; i <= 4; i++) {
                $form.find('[name="IpPart' + i + '"]')
                    .attr('data-initial-val', ipParts[i - 1])
                    .val(ipParts[i - 1]);
            }
        } else {
            $form.find('[name^="IpPart"]')
                .val('')
                .attr('data-initial-val', '');
        }
    }

    function saveIp () {
        $editIpDialog.setDialogState('default');
        var $form = $editIpDialog.find('.reveal-content form');
        // if form not modified and not empty - just close dialog
        if (!formModified($form) && $form.find('input:not(:disabled)').joinVals('') !== '') {
            $editIpDialog.find('[data-close]:first').trigger('click');
            return;
        }
        // validate form
        $form.foundation('validateForm');
    }

    function queryToSaveIp (dsmuid, ip) {
        var $form = $editIpDialog.find('.reveal-content form');
        ajaxRequest('/json/property/vdcquery', {
            query: 'setProperty(deviceId=' + dsmuid + ')/(homematicHostIp="' + ip + '")'
        }, function (data) {
            if (data.ok && data.result) {
                $editIpDialog.setDialogState('success');
                $form.setFormState('hidden');
                $deviceTable.find('[data-dsmuid="' + dsmuid + '"] .ip .rel').html('IP ' + ip);
            } else {
                $editIpDialog.setDialogState('error');
                $form.setFormState('default');
            }
        }, function () {
            $editIpDialog.setDialogState('error');
            $form.setFormState('default');
        }, function () {
            $editIpDialog.find('.button, [data-close]').setState('default');
        });
    }

    /**
     * EDIT WALL BUTTON TYPE
     */

    function openWallButtonTypeDialog () {
        // when, opened, clean up the dialog back to its "initial' state"
        var $tr = $(this).closest('[data-dsuid]');
        var dsuid = $tr.attr('data-dsuid');
        var displayId = $tr.attr('data-display-id');
        var $form = $editWallButtonTypeDialog.find('.reveal-content form');
        var $select = $form.find('select');
        $editWallButtonTypeDialog
            .attr('data-dsuid', dsuid)
            .attr('data-display-id', displayId)
            .setDialogState('default');
        $form
            .setFormState('default')
            .foundation('resetForm');   // clear form and validation
        // fill the form in a modal with to-be-edited dsm data
        var type = $tr.find('.type .rel').text();
        if (type.indexOf('buttonType_') !== -1) {
            $select
                .attr('data-initial-val', type)
            .find('option[value="' + type + '"]')
                .prop('selected', true);
        } else {
            $select
                .attr('data-initial-val', '')
            .find('#SelectButtonType')
                .prop('selected', true);
        }
    }

    function saveWallButtonType () {
        $editWallButtonTypeDialog.setDialogState('default');
        var $form = $editWallButtonTypeDialog.find('.reveal-content form');
        // if form not modified and not empty - just close dialog
        if (!formModified($form) && $form.find('select option:selected').val()) {
            $editWallButtonTypeDialog.find('[data-close]:first').trigger('click');
            return;
        }
        // validate form
        $form.foundation('validateForm');
    }

    function showResultAndRebuildDevicesTable (success) {
        $editWallButtonTypeDialog
            .setDialogState(success ? 'success' : 'error')
        .find('.button, [data-close]')
            .setState('default');

        $deviceTableWorkingCopy.html($deviceTable.html());
        $deviceTable.setTableState('in progress');
        setTimeout(buildTable, reregisterDelay * 2);
    }

    function removeInactivatedDevice (data, dsuid, $select) {
        var success = data && data.ok;
        var prevSelected = $select.attr('data-initial-val');  // value before change

        if (success) {
            // remove inactive device created by change ("new" device gets new dsuid - old one becomes inactive)
            ajaxRequest('/json/structure/removeDevice', {
                dsuid: dsuid
            }, function (data) {
                success = data && data.ok;
                showResultAndRebuildDevicesTable(success);
            }, function () {
                showResultAndRebuildDevicesTable(false);
            });
        } else {
            $select.setSelected(prevSelected);
            showResultAndRebuildDevicesTable(false);
        }
    }

    function whenNewDeviceCreated (data, dsuid, $select) {
        checkCount--;
        var i;
        var len = dsmuids.length;

        ajaxRequest('/json/property/query2', {
            query: '/apartment/zones/zone0/devices/*(DisplayID)'
        }, function (data) {
            if (data && data.ok) {
                var addingCompleted = false;
                for (var key in data.result) {
                    if (data.result[key].DisplayID === $editWallButtonTypeDialog.attr('data-display-id')) {
                        addingCompleted = true;
                        break;
                    }
                }
                if (addingCompleted) {
                    for (i = 0; i < len; i++) {
                        ajaxRequest('/dsa/json/dsm/restartRegistration', { dsuid: dsmuids[i] }, true);
                    }

                    setTimeout(function () {
                        removeInactivatedDevice(data, dsuid, $select);
                    }, reregisterDelay);
                } else if (checkCount > 0) {
                    setTimeout(function () {
                        whenNewDeviceCreated(data, dsuid, $select);
                    }, checkInterval);
                } else {
                    // limit of re-checks reached; try to remove anyway
                    for (i = 0; i < len; i++) {
                        ajaxRequest('/dsa/json/dsm/restartRegistration', { dsuid: dsmuids[i] }, true);
                    }

                    setTimeout(function () {
                        removeInactivatedDevice(data, dsuid, $select);
                    }, reregisterDelay);
                }
            }
        }, function () {
            $editWallButtonTypeDialog.setDialogState('error');
            // $form.setFormState('default');
            $rediscoverButton.setState('default');
            // $editWallButtonTypeDialog.find('.button, [data-close]').setState('default');
        });
    }

    function queryToSaveWallButtonType (dsuid, type) {
        $rediscoverButton.setState('disabled');
        var $form = $editWallButtonTypeDialog.find('.reveal-content form');

        function errorCallback () {
            $editWallButtonTypeDialog.setDialogState('error');
            // $form.setFormState('default');
            $rediscoverButton.setState('default');
            // $editWallButtonTypeDialog.find('.button, [data-close]').setState('default');
        }

        ajaxRequest('/json/property/vdcquery', {
            query: 'setProperty(deviceId=' + dsuid + ')/(hmButtonType="' + type + '")'
        }, function (data) {
            if (data.ok && data.result) {
                // wait till adding new device succeeds
                checkCount = checkLimit;
                setTimeout(function () {
                    whenNewDeviceCreated(data, dsuid, $form.find('select'));
                }, checkInterval);

                // $editWallButtonTypeDialog.setDialogState('success');
                // $form.setFormState('hidden');
                // $deviceTable.find('.type_' + dsuid + ' .rel').html(type);
            } else {
                errorCallback();
            }
        }, function () {
            errorCallback();
        });
    }

    /**
     * CLOUD CONNECTION
     */

    function isCloudAvailable (fRefresh) {
        if (!fRefresh && cloudAvailable !== null) {
            return cloudAvailable;
        }

        cloudAvailable = false;
        var result = ajaxRequest('/cgi-bin/remoteaccessinfo.sh', {});
        if (result && result.success && result.data && typeof result.data.remoteaccess !== 'undefined') {
            cloudAvailable = result.data.remoteaccess;
        }
        return cloudAvailable;
    }

    function reregister () {
        $rediscoverButton.setState('in progress');
        $searchingForDevicesPopup.open();
        $deviceTableWorkingCopy.html($deviceTable.html());
        $deviceTable.setTableState('in progress');

        for (var i = 0, len = dsmuids.length; i < len; i++) {
            ajaxRequest('/dsa/json/dsm/restartRegistration', { dsuid: dsmuids[i] }, true);
        }

        setTimeout(buildTable, reregisterDelay);
    }

    function checkRemoteConnect (initial) {
        vdcToken = null;
        if (!isCloudAvailable(true)) {
            $remoteConnectivityPopup.open();
            $deviceTable.setTableState('disconnected');
            $rediscoverButton.setState('disabled');
        } else {
            ajaxRequest('/json/property/query2', {
                query: '/system/security/vdcToken(value)'
            }, function (tempVdcToken) {
                tempVdcToken = tempVdcToken.result;
                if (tempVdcToken.vdcToken && typeof tempVdcToken.vdcToken.value !== 'undefined') {
                    vdcToken = tempVdcToken.vdcToken.value;
                }

                buildTable(initial);
            }, function () {
                buildTable(initial);
            });
        }
    }

    function reCheckRemoteConnect (e) {
        if (vdcToken === null) {
            e.preventDefault();
            checkRemoteConnect();
        }
    }

    /**
     * INITIALIZE EVENTS
     */

    function initializeDialogEvents () {
        // ip dialog
        var $editIpSubmit = $editIpDialog.find('button[type="submit"]');
        var $editIpForm = $editIpDialog.find('form');
        $editIpSubmit.on('click', saveIp);
        var $editIpFormIpParts = $editIpForm.find('input:not(:disabled)');
        $editIpFormIpParts
            .on('keydown', function (e) {
                if ($.inArray(e.keyCode, [ 46, 8, 9, 27, 13, 110, 190 ]) !== -1 ||
                     // allow: Ctrl+A, Command+A, Ctrl+C, Ctrl+V
                    ((e.keyCode === 65 || e.keyCode === 67 || e.keyCode === 86) && (e.ctrlKey === true || e.metaKey === true)) ||
                     // allow: home, end, left, right, down, up
                    (e.keyCode >= 35 && e.keyCode <= 40) ||
                    // allow Tab, Shift+Tab
                    (e.keyCode === 9)) {
                    // let it happen, don't do anything
                    return;
                }
                // ensure that it is a number and stop the keypress
                if ((e.shiftKey || (e.keyCode < 48 || e.keyCode > 57)) && (e.keyCode < 96 || e.keyCode > 105)) {
                    e.preventDefault();
                }
            })
            .on('paste', function (e) {
                var clipboardData = e.clipboardData || e.originalEvent.clipboardData || window.clipboardData;
                var pastedData = clipboardData.getData('text');
                var cleanedData = '';
                for (var i = 0, len = pastedData.length; i < len; i++) {
                    if (pastedData[i] >= '0' && pastedData[i] <= '9') {
                        cleanedData += pastedData[i];
                    }
                }
                e.clipboardData = e.originalEvent.clipboardData = window.clipboardData = cleanedData;
            });
        $editIpForm
            .on('keyup', '[name^="IpPart"]', function (e) {
                if (e.keyCode === 9 || e.keyCode === 16) {  // allow manual Tab / Shift+Tab action
                    return;
                }
                var maxlength = false;
                if ($(this).attr('maxlength')) {
                    if ($(this).val().length >= $(this).attr('maxlength')) {
                        maxlength = true;
                    }
                }
                if (e.keyCode === 13 || maxlength) {
                    var current = $editIpFormIpParts.index(this);
                    var next = $editIpFormIpParts.eq(current + 1).length ? $editIpFormIpParts.eq(current + 1) : $editIpFormIpParts.eq(0);
                    next.focus();
                }
            })
            .on('formvalid.zf.abide', function () {
                // set modal to "saving state"
                $editIpForm.setFormState('disabled');
                $editIpDialog.find('.button, [data-close]').setState('disabled');
                $editIpSubmit.setState('in progress');
                // fire up query
                queryToSaveIp($editIpDialog.attr('data-dsmuid'), $editIpForm.find('[name^="IpPart"]').joinVals('.'));
            });

        var $editWallButtonTypeSubmit = $editWallButtonTypeDialog.find('button[type="submit"]');
        var $editWallButtonTypeForm = $editWallButtonTypeDialog.find('.reveal-content form');
        $editWallButtonTypeSubmit.on('click', saveWallButtonType);
        $editWallButtonTypeForm
            .on('formvalid.zf.abide', function () {
                // set modal to "saving state"
                $editWallButtonTypeForm.setFormState('disabled');
                $editWallButtonTypeDialog.find('.button, [data-close]').setState('disabled');
                $editWallButtonTypeSubmit.setState('in progress');
                // fire up query
                queryToSaveWallButtonType($editWallButtonTypeDialog.attr('data-dsuid'), $editWallButtonTypeForm.find('select option:selected').val());
            });
    }

    function toggleSubrows () {
        $(this).toggleSubrows($deviceTable);
    }

    function initializeEvents () {
        $(document)
            .on('click', '.recheck-remote-connect', reCheckRemoteConnect)
            .on('click', '#button_rediscover_devices', reregister)
            .on('click', '[data-open="EditIp"]', openIpDialog)
            .on('click', '[data-open="EditWallButtonType"]', openWallButtonTypeDialog)
            .on('click', '.collapsible [data-dsmuid]:not(.no-devices) .name', toggleSubrows);

        initializeDialogEvents();
    }

    $(document).ready(function () {
        buildEnvironment();
        translate();

        checkRemoteConnect(true);

        initializeEvents();
    });

})(jQuery);
